import { P2P as P2PTypes } from '@shardeum-foundation/lib-types';
import * as Account from '../dbstore/accounts';
import * as Transaction from '../dbstore/transactions';
import * as Receipt from '../dbstore/receipts';
import * as OriginalTxsData from '../dbstore/originalTxsData';
import * as State from '../State';
import { DataType, GossipData } from './GossipData';
import * as OriginalTxDB from '../dbstore/originalTxsData';
export declare let storingAccountData: boolean;
export interface ReceiptVerificationResult {
    success: boolean;
    failedReasons?: string[];
    nestedCounterMessages?: string[];
}
/**
 * Validate type and field existence of the receipt data before processing it further
 * @param receipt
 * @returns boolean
 */
export declare const validateReceiptType: (receipt: Receipt.Receipt | Receipt.ArchiverReceipt) => boolean;
export declare const verifyReceiptData: (receipt: Receipt.Receipt | Receipt.ArchiverReceipt, checkReceiptRobust?: boolean) => Promise<{
    success: boolean;
    requiredSignatures?: number;
    newReceipt?: Receipt.ArchiverReceipt | Receipt.Receipt;
}>;
export declare const verifyArchiverReceipt: (receipt: Receipt.ArchiverReceipt | Receipt.Receipt, requiredSignatures: number) => Promise<ReceiptVerificationResult>;
export declare const storeReceiptData: (receipts: Receipt.Receipt[] | Receipt.ArchiverReceipt[], senderInfo?: string, verifyData?: boolean, saveOnlyGossipData?: boolean) => Promise<void>;
export declare const validateCycleData: (cycleRecord: P2PTypes.CycleCreatorTypes.CycleData) => boolean;
export declare const storeCycleData: (cycles?: P2PTypes.CycleCreatorTypes.CycleData[]) => Promise<void>;
interface StoreAccountParam {
    accounts?: Account.AccountsCopy[];
    receipts?: Transaction.Transaction[];
}
export declare const storeAccountData: (restoreData?: StoreAccountParam) => Promise<void>;
export declare const storeOriginalTxData: (originalTxsData?: OriginalTxsData.OriginalTxData[], senderInfo?: string, saveOnlyGossipData?: boolean) => Promise<void>;
interface validateResponse {
    success: boolean;
    reason?: string;
    error?: string;
}
export declare const validateOriginalTxDataSchema: (originalTxData: OriginalTxsData.OriginalTxData) => boolean;
export declare const validateGossipData: (data: GossipData) => validateResponse;
export declare const processGossipData: (gossipdata: GossipData) => void;
export declare const collectMissingTxDataFromArchivers: () => Promise<void>;
export declare const collectMissingReceipts: (senders: string[], txId: string, txTimestamp: number) => Promise<void>;
type QueryTxDataFromArchiversResponse = Receipt.Receipt[] | OriginalTxDB.OriginalTxData[] | null;
export declare const queryTxDataFromArchivers: (archiver: State.ArchiverNodeInfo, txDataType: DataType, txIdList: [
    string,
    number
][]) => Promise<QueryTxDataFromArchiversResponse>;
export declare function cleanOldReceiptsMap(timestamp: number): void;
export declare function cleanOldOriginalTxsMap(timestamp: number): void;
export declare const scheduleMissingTxsDataQuery: () => void;
export {};
