import * as State from './State';
import * as Crypto from './Crypto';
import * as Data from './Data/Data';
import * as NodeList from './NodeList';
import 'node-fetch';
import { P2P as P2PTypes } from '@shardeum-foundation/lib-types';
import { RequestInit, Response } from 'node-fetch';
import { SignedObject } from '@shardeum-foundation/lib-crypto-utils';
export declare enum RequestTypes {
    JOIN = "JOIN",
    ACTIVE = "ACTIVE",
    LEAVE = "LEAVE"
}
export interface ArchiverJoinRequest {
    nodeInfo: State.ArchiverNodeInfo;
    appData: unknown;
    requestType: RequestTypes.JOIN;
    requestTimestamp: number;
    cycleRecord?: P2PTypes.CycleCreatorTypes.CycleRecord;
}
export interface ArchiverActiveRequest {
    nodeInfo: State.ArchiverNodeInfo;
    requestType: RequestTypes.ACTIVE;
}
export interface ArchiverLeaveRequest {
    nodeInfo: State.ArchiverNodeInfo;
    requestType: RequestTypes.LEAVE;
    requestTimestamp: number;
}
export interface FirstNodeInfo {
    nodeInfo: {
        externalIp: string;
        externalPort: number;
        publicKey: string;
    };
}
export interface FirstNodeResponse {
    nodeList: NodeList.ConsensusNodeInfo[];
    joinRequest?: ArchiverJoinRequest & Crypto.SignedMessage;
    dataRequestCycle?: (Data.DataRequest<P2PTypes.CycleCreatorTypes.CycleRecord> & Crypto.TaggedMessage) | number;
    dataRequestStateMetaData?: Data.DataRequest<P2PTypes.SnapshotTypes.StateMetaData> & Crypto.TaggedMessage;
}
export declare function createArchiverJoinRequest(): ArchiverJoinRequest & SignedObject;
export declare function createArchiverActiveRequest(): ArchiverActiveRequest & SignedObject;
export declare function createArchiverLeaveRequest(): ArchiverLeaveRequest & SignedObject;
export declare function postJson(url: string, body: object, timeoutInSecond?: number): Promise<(object & {
    success?: boolean;
}) | null>;
export declare function getJson(url: string, timeoutInSecond?: number): Promise<object | null>;
export declare function get(url: string, timeoutInSecond?: number, opts?: RequestInit): Promise<Response>;
