"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setNestedCountersInstance = exports.nestedCountersInstance = void 0;
const StringifyReduce_1 = require("./StringifyReduce");
const core = require("@shardeum-foundation/lib-crypto-utils");
const DebugMode_1 = require("../DebugMode");
function setNestedCountersInstance(instance) {
    exports.nestedCountersInstance = instance;
}
exports.setNestedCountersInstance = setNestedCountersInstance;
class NestedCounters {
    constructor(server) {
        // this.sectionTimes = {}
        this.eventCounters = new Map();
        this.rareEventCounters = new Map();
        this.infLoopDebug = false;
        this.server = server;
    }
    registerEndpoints() {
        this.server.get('/counts', {
            preHandler: async (_request, reply) => {
                (0, DebugMode_1.isDebugMiddleware)(_request, reply);
            },
        }, (req, res) => {
            let outputStr = '';
            const arrayReport = this.arrayitizeAndSort(this.eventCounters);
            outputStr += `${Date.now()}\n`;
            outputStr = this.printArrayReport(arrayReport, outputStr, 0);
            res.send(outputStr);
        });
        this.server.get('/counts-reset', {
            preHandler: async (_request, reply) => {
                (0, DebugMode_1.isDebugMiddleware)(_request, reply);
            },
        }, (req, res) => {
            this.eventCounters = new Map();
            res.send(`counts reset ${Date.now()}`);
        });
        this.server.get('/debug-inf-loop', {
            preHandler: async (_request, reply) => {
                (0, DebugMode_1.isDebugMiddleware)(_request, reply);
            },
        }, (req, res) => {
            res.send('starting inf loop, goodbye');
            this.infLoopDebug = true;
            while (this.infLoopDebug) {
                const s = 'asdf';
                const s2 = (0, StringifyReduce_1.stringifyReduce)({ test: [s, s, s, s, s, s, s] });
                const s3 = (0, StringifyReduce_1.stringifyReduce)({ test: [s2, s2, s2, s2, s2, s2, s2] });
                core.hash(s3);
            }
        });
        this.server.get('/debug-inf-loop-off', {
            preHandler: async (_request, reply) => {
                (0, DebugMode_1.isDebugMiddleware)(_request, reply);
            },
        }, (req, res) => {
            this.infLoopDebug = false;
            res.send('stopping inf loop, who knows if this is possible');
        });
    }
    countEvent(category1, category2, count = 1) {
        let counterMap = this.eventCounters;
        let nextNode;
        if (counterMap.has(category1) === false) {
            nextNode = { count: 0, subCounters: new Map() };
            counterMap.set(category1, nextNode);
        }
        else {
            nextNode = counterMap.get(category1);
        }
        nextNode.count += count;
        counterMap = nextNode.subCounters;
        //unrolled loop to avoid memory alloc
        category1 = category2;
        if (counterMap.has(category1) === false) {
            nextNode = { count: 0, subCounters: new Map() };
            counterMap.set(category1, nextNode);
        }
        else {
            nextNode = counterMap.get(category1);
        }
        nextNode.count += count;
        counterMap = nextNode.subCounters;
    }
    countRareEvent(category1, category2, count = 1) {
        // trigger normal event counter
        this.countEvent(category1, category2, count);
        // start counting rare event
        let counterMap = this.rareEventCounters;
        let nextNode = { count: 0, subCounters: new Map() };
        if (!counterMap.has(category1)) {
            nextNode = { count: 0, subCounters: new Map() };
            counterMap.set(category1, nextNode);
        }
        else {
            nextNode = counterMap.get(category1);
        }
        nextNode.count += count;
        counterMap = nextNode.subCounters;
        //unrolled loop to avoid memory alloc
        category1 = category2;
        if (counterMap.has(category1) === false) {
            nextNode = { count: 0, subCounters: new Map() };
            counterMap.set(category1, nextNode);
        }
        else {
            nextNode = counterMap.get(category1);
        }
        nextNode.count += count;
        counterMap = nextNode.subCounters;
    }
    arrayitizeAndSort(counterMap) {
        const array = [];
        for (const key of counterMap.keys()) {
            const valueObj = counterMap.get(key);
            const newValueObj = { key, count: valueObj.count, subArray: null };
            // newValueObj.key = key
            array.push(newValueObj);
            let subArray = [];
            if (valueObj.subCounters != null) {
                subArray = this.arrayitizeAndSort(valueObj.subCounters);
            }
            // if (valueObj.count != null && valueObj.logLen != null) {
            //   valueObj.avgLen = valueObj.logLen / valueObj.count
            // }
            newValueObj.subArray = subArray;
            // delete valueObj['subCounters']
        }
        array.sort((a, b) => b.count - a.count);
        return array;
    }
    printArrayReport(arrayReport, outputStr, indent = 0) {
        const indentText = '___'.repeat(indent);
        for (const item of arrayReport) {
            const { key, count, subArray } = item;
            const countStr = `${count}`;
            outputStr += `${countStr.padStart(10)} ${indentText} ${key}\n`;
            if (subArray != null && subArray.length > 0) {
                outputStr = this.printArrayReport(subArray, outputStr, indent + 1);
            }
        }
        return outputStr;
    }
}
exports.default = NestedCounters;
//# sourceMappingURL=nestedCounters.js.map