"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.verifyGlobalTxAccountChange = exports.InternalTXType = void 0;
const calculateAccountHash_1 = require("./calculateAccountHash");
// Refer to https://github.com/shardeum/shardeum/blob/89db23e1d4ffb86b4353b8f37fb360ea3cd93c5b/src/shardeum/shardeumTypes.ts#L87-L88
var InternalTXType;
(function (InternalTXType) {
    InternalTXType[InternalTXType["SetGlobalCodeBytes"] = 0] = "SetGlobalCodeBytes";
    InternalTXType[InternalTXType["InitNetwork"] = 1] = "InitNetwork";
    InternalTXType[InternalTXType["NodeReward"] = 2] = "NodeReward";
    InternalTXType[InternalTXType["ChangeConfig"] = 3] = "ChangeConfig";
    InternalTXType[InternalTXType["ApplyChangeConfig"] = 4] = "ApplyChangeConfig";
    InternalTXType[InternalTXType["SetCertTime"] = 5] = "SetCertTime";
    InternalTXType[InternalTXType["Stake"] = 6] = "Stake";
    InternalTXType[InternalTXType["Unstake"] = 7] = "Unstake";
    InternalTXType[InternalTXType["InitRewardTimes"] = 8] = "InitRewardTimes";
    InternalTXType[InternalTXType["ClaimReward"] = 9] = "ClaimReward";
    InternalTXType[InternalTXType["ChangeNetworkParam"] = 10] = "ChangeNetworkParam";
    InternalTXType[InternalTXType["ApplyNetworkParam"] = 11] = "ApplyNetworkParam";
    InternalTXType[InternalTXType["Penalty"] = 12] = "Penalty";
})(InternalTXType = exports.InternalTXType || (exports.InternalTXType = {}));
const verifyGlobalTxAccountChange = (receipt, failedReasons = [], nestedCounterMessages = []) => {
    try {
        const signedReceipt = receipt.signedReceipt;
        const internalTx = signedReceipt.tx.value;
        if (internalTx.internalTXType === InternalTXType.InitNetwork) {
            // Refer to https://github.com/shardeum/shardeum/blob/89db23e1d4ffb86b4353b8f37fb360ea3cd93c5b/src/index.ts#L2334
            // no need to do anything, as it is network account creation
            return true;
        }
        else if (internalTx.internalTXType === InternalTXType.ApplyChangeConfig ||
            internalTx.internalTXType === InternalTXType.ApplyNetworkParam) {
            if (signedReceipt.tx.addressHash !== '') {
                for (const account of receipt.beforeStates) {
                    if (account.accountId !== signedReceipt.tx.address) {
                        failedReasons.push(`Unexpected account found in before accounts ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
                        nestedCounterMessages.push(`Unexpected account found in before accounts`);
                        return false;
                    }
                    const expectedAccountHash = signedReceipt.tx.addressHash;
                    const calculatedAccountHash = (0, calculateAccountHash_1.accountSpecificHash)(account.data);
                    if (expectedAccountHash !== calculatedAccountHash) {
                        failedReasons.push(`Account hash before does not match in globalModification tx - ${account.accountId} , ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
                        nestedCounterMessages.push(`Account hash before does not match in globalModification tx`);
                        return false;
                    }
                }
            }
            for (const account of receipt.afterStates) {
                if (account.accountId !== signedReceipt.tx.address) {
                    failedReasons.push(`Unexpected account found in accounts ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
                    nestedCounterMessages.push(`Unexpected account found in accounts`);
                    return false;
                }
                const networkAccountBefore = receipt.beforeStates.find((bAccount) => bAccount?.accountId === account.accountId);
                const networkAccountAfter = receipt.afterStates.find((fAccount) => fAccount?.accountId === signedReceipt.tx.address);
                if (!networkAccountBefore || !networkAccountAfter) {
                    failedReasons.push(`No network account found in accounts ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
                    nestedCounterMessages.push(`No network account found in accounts`);
                    return false;
                }
                networkAccountBefore.data.listOfChanges?.push(internalTx.change);
                networkAccountBefore.data.timestamp = signedReceipt.tx.when;
                const expectedAccountHash = networkAccountAfter.hash;
                console.dir(networkAccountBefore, { depth: null });
                const calculatedAccountHash = (0, calculateAccountHash_1.accountSpecificHash)(networkAccountBefore.data);
                if (expectedAccountHash !== calculatedAccountHash) {
                    failedReasons.push(`Account hash does not match in globalModification tx - ${networkAccountAfter.accountId} , ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
                    nestedCounterMessages.push(`Account hash does not match in globalModification tx`);
                    return false;
                }
            }
            return true;
        }
        else {
            failedReasons.push(`Unexpected internal transaction type in the globalModification tx ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
            nestedCounterMessages.push(`Unexpected internal transaction type in the globalModification tx`);
            return false;
        }
    }
    catch (error) {
        console.error(`verifyGlobalTxAccountChange error`, error);
        failedReasons.push(`Error while verifying global account change ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}, ${error}`);
        nestedCounterMessages.push(`Error while verifying global account change`);
        return false;
    }
};
exports.verifyGlobalTxAccountChange = verifyGlobalTxAccountChange;
//# sourceMappingURL=verifyGlobalTxReceipt.js.map