/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import { Readable } from 'stream';
import { EventEmitter } from 'events';
export declare let statisticsInstance: Statistics;
interface Statistics {
    intervalDuration: number;
    context: unknown;
    counterDefs: string[];
    watcherDefs: {
        [key: string]: () => unknown;
    };
    timerDefs: string[];
    manualStatDefs: string[];
    interval: NodeJS.Timeout | null;
    snapshotWriteFns: Array<() => string>;
    stream: Readable | null;
    streamIsPushable: boolean;
    counters: {
        [key: string]: CounterRing;
    };
    watchers: {
        [key: string]: WatcherRing;
    };
    timers: {
        [key: string]: TimerRing;
    };
    manualStats: {
        [name: string]: ManualRing;
    };
}
interface Config {
    interval?: number;
    save?: boolean;
}
declare class Statistics extends EventEmitter {
    constructor(baseDir: string, config: Config, { counters, watchers, timers, manualStats, }: {
        counters: string[];
        watchers: {
            [key: string]: () => unknown;
        };
        timers: string[];
        manualStats: string[];
    }, context: unknown);
    initialize(): void;
    getStream(): Readable | null;
    writeOnSnapshot(writeFn: () => string, context: unknown): void;
    startSnapshots(): void;
    stopSnapshots(): void;
    incrementCounter(counterName: string): void;
    setManualStat(manualStatName: string, value: number): void;
    getCurrentCount(counterName: string): number;
    getCounterTotal(counterName: string): number;
    getWatcherValue(watcherName: string): unknown;
    startTimer(timerName: string, id: string): void;
    stopTimer(timerName: string, id: string): void;
    getAverage(name: string): number;
    getMultiStatReport(name: string): {
        min: number;
        max: number;
        avg: number;
        allVals: number[];
    };
    getPreviousElement(name: string): unknown;
    _initializeCounters(counterDefs?: string[]): {
        [key: string]: CounterRing;
    };
    _initializeWatchers(watcherDefs: {
        [key: string]: () => unknown;
    }, context: unknown): {
        [key: string]: WatcherRing;
    };
    _initializeTimers(timerDefs?: string[]): {
        [key: string]: TimerRing;
    };
    _initializeManualStats(counterDefs?: string[]): {
        [key: string]: ManualRing;
    };
    _takeSnapshot(): void;
    _pushToStream(data: unknown): void;
}
interface Ring {
    elements: unknown[];
    index: number;
}
declare class Ring {
    constructor(length: number);
    multiStats(): {
        min: number;
        max: number;
        avg: number;
        allVals: number[];
    };
    save(value: unknown): void;
    average(): number;
    previous(): number;
}
interface CounterRing {
    count: number;
    total: number;
    ring: Ring;
}
declare class CounterRing {
    constructor(length: number);
    increment(): void;
    snapshot(): void;
}
interface WatcherRing {
    watchFn: () => unknown;
    ring: Ring;
}
declare class WatcherRing {
    constructor(length: number, watchFn: (...args: unknown[]) => unknown, context: unknown);
    snapshot(): void;
}
interface TimerRing {
    ids: Record<string, number>;
    ring: Ring;
}
declare class TimerRing {
    constructor(length: number);
    start(id: string): void;
    stop(id: string): void;
    snapshot(): void;
}
interface ManualRing {
    ring: Ring;
}
declare class ManualRing {
    constructor(length: number);
    manualSetValue(value: unknown): void;
    snapshot(): void;
}
export default Statistics;
