"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTxDigestsForACycleRange = exports.processAndInsertTxDigests = exports.getLastProcessedTxDigest = exports.updateLastProcessedTxDigest = exports.getHash = exports.getTxIds = void 0;
const Config_1 = require("../Config");
const processedTxs = require("../dbstore/processedTxs");
const txDigest = require("./txDigests");
const Crypto = require("../Crypto");
let lastProcessedTxDigest = null;
const getTxIds = async (startCycle, endCycle) => {
    const sortedTxIds = await processedTxs.querySortedTxsBetweenCycleRange(startCycle, endCycle);
    return sortedTxIds;
};
exports.getTxIds = getTxIds;
const getHash = async (cycle) => {
    if (cycle == -1) {
        return '0x0';
    }
    const txDigestHash = await txDigest.queryByEndCycle(cycle);
    if (!txDigestHash) {
        throw new Error(`Failed to fetch txDigestHash for cycle ${cycle}`);
    }
    return txDigestHash.hash;
};
exports.getHash = getHash;
const updateLastProcessedTxDigest = async () => {
    lastProcessedTxDigest = await txDigest.getLastProcessedTxDigest();
};
exports.updateLastProcessedTxDigest = updateLastProcessedTxDigest;
const getLastProcessedTxDigest = async () => {
    if (!lastProcessedTxDigest) {
        await (0, exports.updateLastProcessedTxDigest)();
    }
    return lastProcessedTxDigest;
};
exports.getLastProcessedTxDigest = getLastProcessedTxDigest;
const processAndInsertTxDigests = async (lastCheckedCycle, latestCycleCounter) => {
    console.log('Processing and inserting txDigests from cycle: ', lastCheckedCycle, ' to ', latestCycleCounter);
    const batchSize = Config_1.config.txDigest.cycleDiff;
    let currentCycle = lastCheckedCycle;
    let endCycle = currentCycle + batchSize - 1;
    while (endCycle <= latestCycleCounter) {
        console.log(`Processing txDigests from cycle ${currentCycle} to ${endCycle}`);
        // Fetch txDigests in the current batch
        const txIds = await (0, exports.getTxIds)(currentCycle, endCycle);
        if (txIds == null) {
            console.error(`Failed to fetch txIds for cycle ${currentCycle} to ${endCycle}`);
            return;
        }
        if (Config_1.config.VERBOSE) {
            console.log(`TxIds from ${currentCycle} to ${endCycle} of length ${txIds.length}: `, txIds);
        }
        const prevHash = await (0, exports.getHash)(currentCycle - 1);
        console.log(`prevHash for cycle ${currentCycle}: `, prevHash);
        const txObj = {
            prevHash: prevHash,
            txIdsHash: Crypto.hashObj(txIds),
        };
        const txRangeHash = Crypto.hashObj(txObj);
        const txDigestObj = {
            cycleStart: currentCycle,
            cycleEnd: endCycle,
            txCount: txIds.length,
            hash: txRangeHash,
        };
        try {
            txDigest.insertTransactionDigest(txDigestObj);
        }
        catch (e) {
            console.error('Failed to insert txDigestObj: ', txDigestObj);
            console.error(e);
            return;
        }
        currentCycle = endCycle + 1;
        endCycle = currentCycle + batchSize - 1;
        lastProcessedTxDigest = txDigestObj;
    }
    console.log('Updated lastProcessedTxDigest: ', lastProcessedTxDigest);
    console.log('Finished processing txDigests.');
};
exports.processAndInsertTxDigests = processAndInsertTxDigests;
const getTxDigestsForACycleRange = async (cycleStart, cycleEnd) => {
    const txDigests = await txDigest.queryByCycleRange(cycleStart, cycleEnd);
    return txDigests;
};
exports.getTxDigestsForACycleRange = getTxDigestsForACycleRange;
//# sourceMappingURL=txDigestFunctions.js.map