"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryByCycleRange = exports.queryByEndCycle = exports.getLastProcessedTxDigest = exports.insertTransactionDigest = void 0;
const db = require("../dbstore/sqlite3storage");
const _1 = require(".");
const Config_1 = require("../Config");
async function insertTransactionDigest(txDigest) {
    try {
        const fields = Object.keys(txDigest).join(', ');
        const placeholders = Object.keys(txDigest).fill('?').join(', ');
        const values = db.extractValues(txDigest);
        const sql = 'INSERT INTO txDigests (' +
            fields +
            ') VALUES (' +
            placeholders +
            ') ON CONFLICT (cycleEnd) DO UPDATE SET ' +
            'cycleStart = excluded.cycleStart, ' +
            'txCount = excluded.txCount, ' +
            'hash = excluded.hash';
        await db.run(_1.digesterDatabase, sql, values);
        if (Config_1.config.VERBOSE) {
            console.log(`Successfully inserted txDigest for cycle records from ${txDigest.cycleStart} to ${txDigest.cycleEnd}`);
        }
    }
    catch (e) {
        console.error(e);
        throw new Error(`Unable to insert txDigest for cycle records from ${txDigest.cycleStart} to ${txDigest.cycleEnd}`);
    }
}
exports.insertTransactionDigest = insertTransactionDigest;
async function getLastProcessedTxDigest() {
    try {
        const sql = `SELECT * FROM txDigests ORDER BY cycleEnd DESC LIMIT 1`;
        const lastProcessedDigest = (await db.get(_1.digesterDatabase, sql));
        if (Config_1.config.VERBOSE) {
            console.log('LastProcessed Tx Digest', lastProcessedDigest);
        }
        return lastProcessedDigest;
    }
    catch (e) {
        console.error(e);
        return null;
    }
}
exports.getLastProcessedTxDigest = getLastProcessedTxDigest;
async function queryByEndCycle(endCycle) {
    try {
        const sql = `SELECT * FROM txDigests WHERE cycleEnd=? LIMIT 1`;
        const txDigest = (await db.get(_1.digesterDatabase, sql, [endCycle]));
        if (Config_1.config.VERBOSE) {
            console.log('Tx Digest by endCycle', txDigest);
        }
        return txDigest;
    }
    catch (e) {
        console.error(e);
        return null;
    }
}
exports.queryByEndCycle = queryByEndCycle;
async function queryByCycleRange(startCycle, endCycle) {
    try {
        const sql = `SELECT * FROM txDigests WHERE cycleStart >= ? AND cycleEnd <= ? ORDER BY cycleEnd`;
        const txDigests = (await db.all(_1.digesterDatabase, sql, [startCycle, endCycle]));
        if (Config_1.config.VERBOSE) {
            console.log('Tx Digest by cycle range', txDigests);
        }
        return txDigests || [];
    }
    catch (e) {
        console.error(e);
        return [];
    }
}
exports.queryByCycleRange = queryByCycleRange;
//# sourceMappingURL=txDigests.js.map